@@ -377,7 +377,7 @@ lpaws <- function(y,
           .Fortran(C_awsph1,
             as.double(y),
             as.double(sigma2),
-            fix = as.logical(fix),
+            fix = as.integer(fix),
             as.integer(nfix),
             as.integer(n),
             as.integer(degree),
@@ -405,7 +405,7 @@ lpaws <- function(y,
           .Fortran(C_awsph2,
             as.double(y),
             as.double(sigma2),
-            fix = as.logical(fix),
+            fix = as.integer(fix),
             as.integer(nfix),
             as.integer(n1),
             as.integer(n2),
@@ -438,7 +438,7 @@ lpaws <- function(y,
           d,
           .Fortran(C_awsp1b,
             as.double(y),
-            fix = as.logical(fix),
+            fix = as.integer(fix),
             as.integer(nfix),
             as.integer(n),
             as.integer(degree),
@@ -465,7 +465,7 @@ lpaws <- function(y,
           )[c("bi", "bi0", "bi2", "ai", "hakt", "hhom", "fix")],
           .Fortran(C_awsp2,
             as.double(y),
-            fix = as.logical(fix),
+            fix = as.integer(fix),
             as.integer(nfix),
             as.integer(n1),
             as.integer(n2),
@@ -504,6 +504,7 @@ lpaws <- function(y,
       if (!homogen)
         zobj$hhom <- switch(d, rep(1, 2 * n), rep(1, n))
       dim(fix) <- switch(d, n, dy)
+      zobj$fix <- as.logical(zobj$fix)
       tobj <- updtheta(d, zobj, fix, cpar, aggkern, bikm1, bi2km1, thetakm1)
       dim(bikm1) <- dim(bi2km1) <- dim(bi) <- dim(bi2) <- c(n, dp2)
       dim(thetakm1) <- dim(theta) <- c(n, dp1)
